@@ -591,7 +591,7 @@ static void snd_pcm_shm_dump(snd_pcm_t *pcm, snd_output_t *out)
 	}
 }
 
-static snd_pcm_ops_t snd_pcm_shm_ops = {
+static const snd_pcm_ops_t snd_pcm_shm_ops = {
 	.close = snd_pcm_shm_close,
 	.info = snd_pcm_shm_info,
 	.hw_refine = snd_pcm_shm_hw_refine,
@@ -606,7 +606,7 @@ static snd_pcm_ops_t snd_pcm_shm_ops = {
 	.munmap = snd_pcm_shm_munmap,
 };
 
-static snd_pcm_fast_ops_t snd_pcm_shm_fast_ops = {
+static const snd_pcm_fast_ops_t snd_pcm_shm_fast_ops = {
 	.status = snd_pcm_shm_status,
 	.state = snd_pcm_shm_state,
 	.hwsync = snd_pcm_shm_hwsync,
@@ -654,33 +654,6 @@ static int make_local_socket(const char *filename)
 	return sock;
 }
 
-#if 0
-static int make_inet_socket(const char *host, int port)
-{
-	struct sockaddr_in addr;
-	int sock;
-	struct hostent *h = gethostbyname(host);
-	if (!h)
-		return -ENOENT;
-
-	sock = socket(PF_INET, SOCK_STREAM, 0);
-	if (sock < 0) {
-		SYSERR("socket failed");
-		return -errno;
-	}
-	
-	addr.sin_family = AF_INET;
-	addr.sin_port = htons(port);
-	memcpy(&addr.sin_addr, h->h_addr_list[0], sizeof(struct in_addr));
-
-	if (connect(sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		SYSERR("connect failed");
-		return -errno;
-	}
-	return sock;
-}
-#endif
-
 /**
  * \brief Creates a new shared memory PCM
  * \param pcmp Returns created PCM handle
@@ -842,12 +815,10 @@ int _snd_pcm_shm_open(snd_pcm_t **pcmp, const char *name,
 	const char *server = NULL;
 	const char *pcm_name = NULL;
 	snd_config_t *sconfig;
-	const char *host = NULL;
 	const char *sockname = NULL;
 	long port = -1;
 	int err;
-	int local;
-	struct hostent *h;
+
 	snd_config_for_each(i, next, conf) {
 		snd_config_t *n = snd_config_iterator_entry(i);
 		const char *id;
@@ -898,14 +869,8 @@ int _snd_pcm_shm_open(snd_pcm_t **pcmp, const char *name,
 			continue;
 		if (strcmp(id, "comment") == 0)
 			continue;
-		if (strcmp(id, "host") == 0) {
-			err = snd_config_get_string(n, &host);
-			if (err < 0) {
-				SNDERR("Invalid type for %s", id);
-				goto _err;
-			}
+		if (strcmp(id, "host") == 0)
 			continue;
-		}
 		if (strcmp(id, "socket") == 0) {
 			err = snd_config_get_string(n, &sockname);
 			if (err < 0) {
@@ -928,24 +893,10 @@ int _snd_pcm_shm_open(snd_pcm_t **pcmp, const char *name,
 		goto __error;
 	}
 
-	if (!host) {
-		SNDERR("host is not defined");
-		goto _err;
-	}
 	if (!sockname) {
 		SNDERR("socket is not defined");
 		goto _err;
 	}
-	h = gethostbyname(host);
-	if (!h) {
-		SNDERR("Cannot resolve %s", host);
-		goto _err;
-	}
-	local = snd_is_local(h);
-	if (!local) {
-		SNDERR("%s is not the local host", host);
-		goto _err;
-	}
 	err = snd_pcm_shm_open(pcmp, name, sockname, pcm_name, stream, mode);
       __error:
 	snd_config_delete(sconfig);
